@@ -1,4 +1,6 @@
 # PiFlash::Inspector - inspection of the Linux system configuration including identifying SD card devices
+# by Ian Kluft
+
 use strict;
 use warnings;
 use v5.18.0; # require 2014 or newer version of Perl
@@ -6,10 +8,11 @@ use PiFlash::State;
 use PiFlash::Command;
 
 package PiFlash::Inspector;
-$PiFlash::Inspector::VERSION = '0.0.4';
+$PiFlash::Inspector::VERSION = '0.0.5';
 use autodie; # report errors instead of silently continuing ("die" actions are used as exceptions - caught & reported)
 use File::Basename;
-use File::Slurp;
+use File::Slurp qw(slurp);
+use File::LibMagic; # rpm: "dnf install perl-File-LibMagic", deb: "apt-get install libfile-libmagic-perl"
 
 # ABSTRACT: PiFlash functions to inspect Linux system devices to flash an SD card for Raspberry Pi
 
@@ -59,6 +62,7 @@ sub collect_system_info
 	# find filesystems supported by this kernel (for formatting SD card)
 	my %fs_pref = (vfat => 1, ext4 => 2, ext3 => 3, ext2 => 4, exfat => 5, other => 6); # fs preference order
 	my @filesystems = grep {! /^nodev\s/} slurp("/proc/filesystems");
+	chomp @filesystems;
 	for (my $i=0; $i<=$#filesystems; $i++) {
 		# remove leading and trailing whitespace;
 		$filesystems[$i] =~ s/^\s*//;
@@ -213,6 +217,10 @@ sub collect_device_info
 	if ($output->{mountpoint} ne "") {
 		PiFlash::State->error("output device is mounted - this operation would erase it");
 	}
+	if (!(exists $output->{fstype}) or $output->{fstype} =~ /^\s*$/) {
+		# workaround for apparent bug in lsblk in util-linux which omits requested FSTYPE data
+		$output->{fstype} = get_fstype($output->{path}) // "";
+	}
 	if ($output->{fstype} eq "swap") {
 		PiFlash::State->error("output device is a swap device - this operation would erase it");
 	}
@@ -264,6 +272,7 @@ sub blkparam
 		} elsif ($? != 0) {
 			PiFlash::State->error(sprintf "blkparam($paramname): lsblk exited with value %d", $? >> 8);
 		}
+		chomp $value;
 		$value =~ s/^\s*//; # remove leading whitespace
 		$value =~ s/\s*$//; # remove trailing whitespace
 		$blkdev->{lc $paramname} = $value;
@@ -308,6 +317,7 @@ sub is_sd
 				.PiFlash::State::system("release")." may be too old");
 		}
 		my $sysfs_devtype = slurp($sysfs_devtype_path);
+		chomp $sysfs_devtype;
 		PiFlash::State::verbose() and say "output device ".$blkdev->{path}." is a $sysfs_devtype";
 		if ($sysfs_devtype eq "SD") {
 			return 1;
@@ -365,24 +375,47 @@ sub base
 	return $filename;
 }
 
-# slurp function: get file contents into a string
-sub slurp
+# get filesystem type info
+# workaround for apparent bug in lsblk (from util-linux) which omits requested FSTYPE data when in the background
+# use blkid or libmagic if it fails
+sub get_fstype
 {
-	my $filename = shift;
-	open my $fh, "<", $filename;
-	my $str = do {
-		## no critic (RequireInitializationForLocalVars)
-		local $/; # avoid interference from functions which modify global $/
-		<$fh>
-	};
-	close $fh;
-	if (wantarray) {
-		my @str = split /^/, $str;
-		chomp @str;
-		return @str;
+	my $devpath = shift;
+	my $fstype = PiFlash::Command::cmd2str( "use lsblk to get fs type for $devpath", PiFlash::Command::prog("sudo"),
+		PiFlash::Command::prog("lsblk"), "--nodeps", "--noheadings", "--output", "FSTYPE", $devpath);
+
+	# fallback: use blkid
+	if ((!defined $fstype) or $fstype =~ /^\s*$/) {
+		$fstype = PiFlash::Command::cmd2str( "use blkid to get fs type for $devpath", PiFlash::Command::prog("sudo"),
+			PiFlash::Command::prog("blkid"), "--probe", "--output=value", "--match-tag=TYPE", $devpath);
+
+		# fallback: use File::LibMagic as backup filesystem type lookup 
+		if ((!defined $fstype) or $fstype =~ /^\s*$/) {
+			my $magic = File::LibMagic->new();
+			$fstype = undef;
+			$magic->{flags} |= File::LibMagic::MAGIC_DEVICES; # undocumented trick for equivalent of "file -s" on device
+			my $magic_data = $magic->info_from_filename($devpath);
+			if (PiFlash::State::verbose()) {
+				for my $key (keys %$magic_data) {
+					say "get_fstype: magic_data/$key = ".$magic_data->{$key};
+				}
+			}
+			if ($magic_data->{description} =~ /^Linux rev \d+.\d+ (ext[234]) filesystem data,/) {
+				$fstype=$1;
+			} elsif ($magic_data->{description} =~ /^DOS\/MBR boot sector, .*, OEM-ID "mkfs.fat",.*, FAT (32 bit),/) {
+				$fstype="vfat";
+			} elsif ($magic_data->{description} =~ /^Linux\/\w+ swap file/) {
+				$fstype="swap";
+			} elsif ($magic_data->{description} =~ /\s+(\w+)\sfilesystem/i) {
+				$fstype=lc $1;
+			}
+		}
 	}
-	chomp $str;
-	return $str;
+
+	# lookup failure if we get here
+	defined $fstype and chomp $fstype;
+	PiFlash::State::verbose() and say "get_fstype($devpath) = ".($fstype // "undef");
+	return $fstype;
 }
 
 1;
@@ -399,7 +432,7 @@ PiFlash::Inspector - PiFlash functions to inspect Linux system devices to flash
 
 =head1 VERSION
 
-version 0.0.4
+version 0.0.5
 
 =head1 SYNOPSIS
 
@@ -417,7 +450,7 @@ This class contains internal functions used by L<PiFlash> in the process of coll
 
 =head1 SEE ALSO
 
-L<piflash>, L<PiFlash::Command>, L<PiFlash::Inspector>, L<PiFlash::State>
+L<piflash>, L<PiFlash::Command>, L<PiFlash::State>
 
 =head1 AUTHOR
 
